==> blancoNLpackGenerator-0.0.34-src.zip: src/blanco/nlpack/generator/csv/io/BlancoNLpackGeneratorResultDetailCsvReader.java <==
/*
 * ̃\[XR[h blanco FrameworkɂĎĂ܂B
 */
package blanco.nlpack.generator.csv.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import blanco.nlpack.generator.csv.record.BlancoNLpackGeneratorResultDetailCsvRecord;
import blanco.nlpack.generator.csv.util.BlancoCsvRuntimeUtil;

/**
 * t@C`[BlancoNLpackGeneratorResultDetail/blancoNLpackGenerator IɗpCSVt@CB]̃[_NX
 *
 * ̃NXblancoCsvt@C`玩ꂽ[_NXłB<br>
 * @1.񒷃`FbNɂ́AftHgGR[fBO𗘗p܂B<br>
 * @2.NX̗p͕K close()ĂяoĂB<br>
 */
public class BlancoNLpackGeneratorResultDetailCsvReader {
    /**
     * [_IuWFNg
     *
     * IɎۂɓ͂s̃[_L܂B
     */
    private BufferedReader fReader;

    /**
     * ̍sJE^
     *
     * ݏsĂs̈ʒuL܂B
     */
    private long fLineCounter = 0;

    /**
     * ݏ̍sf[^
     */
    private String fLine;

    /**
     * ݏ̍ŝ߂̃[_
     */
    private StringReader fLineReader;

    /**
     * CSV[_NX̃RXgN^
     *
     * ^ꂽ[_ƘA܂B
     *
     * @param arg A郊[_
     */
    public BlancoNLpackGeneratorResultDetailCsvReader(final BufferedReader arg) {
        fReader = arg;
    }

    /**
     * ̈sǂݍ݂܂
     *
     * @return sIuWFNgԂ܂B͂sꍇɂ nullԂ܂B
     * @throws BlancoCsvIOException ̓f[^sȏꍇȂǁB
     * @throws IOException A惊[_ňُ킪ꍇB
     */
    public BlancoNLpackGeneratorResultDetailCsvRecord readRecord() throws BlancoCsvIOException, IOException {
        fLine = fReader.readLine();
        if (fLine == null) {
            // t@C̏I[ɓB܂B
            return null;
        }
        fLineCounter++;
        final BlancoNLpackGeneratorResultDetailCsvRecord record = new BlancoNLpackGeneratorResultDetailCsvRecord();
        fLineReader = new StringReader(fLine);
        String tokenString = null;

        // ڔԍ[1]ږ[BundleUri/URI]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 1ږځBڔԍ[1] ږ[BundleUri/URI]̏ɂĕsȏI[m܂B");
        }
        // K{ځB
        if (tokenString.length() == 0) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 1ږځBڔԍ[1] ږ[BundleUri/URI]̏ɂĕK{ڂɒlĂȂƂm܂B");
        }
        record.setBundleUri(tokenString);

        // ڔԍ[2]ږ[Action/Action]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 2ږځBڔԍ[2] ږ[Action/Action]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setAction(tokenString);
        } else {
            if (tokenString.getBytes().length < 3) {
                throw new BlancoCsvIOException("" + fLineCounter + "s 2ږځBڔԍ[2] ږ[Action/Action]̏ɂMIN(3)Zl[" + tokenString + "]m܂B");
            }
            record.setAction(tokenString);
        }

        // ڔԍ[3]ږ[Key/Key]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 3ږځBڔԍ[3] ږ[Key/Key]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setKey(tokenString);
        } else {
            if (tokenString.getBytes().length < 1) {
                throw new BlancoCsvIOException("" + fLineCounter + "s 3ږځBڔԍ[3] ږ[Key/Key]̏ɂMIN(1)Zl[" + tokenString + "]m܂B");
            }
            record.setKey(tokenString);
        }

        // ڔԍ[4]ږ[Input/Input]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 4ږځBڔԍ[4] ږ[Input/Input]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setInput(tokenString);
        } else {
            record.setInput(tokenString);
        }

        // ڔԍ[5]ږ[Output/Output]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', true);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 5ږځBڔԍ[5] ږ[Output/Output]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setOutput(tokenString);
        } else {
            record.setOutput(tokenString);
        }
        return record;
    }

    /**
     * IɕێĂ郊[_Ԃ܂B
     *
     * @return IɕێĂ郊[_B
     */
    public BufferedReader getReader() {
        return fReader;
    }

    /**
     * ݏ̍sJE^Ԃ܂B
     *
     * @return ݏ̍sJE^B
     */
    public long getLineCounter() {
        return fLineCounter;
    }

    /**
     * ̃[_܂B
     *
     * ֘AÂĂ[_ɑ΂Ăclose()Ăяo܂B
     *
     * @throws IOException ֘AÂĂ[_close()ɎsꍇB
     */
    public void close() throws IOException {
        fReader.close();
    }

    /**
     * ^ꂽReader ^ꂽf~^găg[No܂
     *
     * @param delimiter f~^
     * @param isEndOfLine s̏I[ł̂ǂtO
     * @return oꂽg[NB͂⃊[_̏ꍇɂnullԂ܂B
     * @throws BlancoCsvIOException ڐȂȂǗ^ꂽɊւOꍇB
     * @throws IOException o͗OꍇB
     */
    protected String readToken(final char delimiter, final boolean isEndOfLine) throws BlancoCsvIOException, IOException {
        final StringBuffer buffer = new StringBuffer();
        for (;;) {
            final int iRead = fLineReader.read();
            if (iRead < 0) {
                // I[ɓBB
                if (isEndOfLine == false) {
                    throw new BlancoCsvIOException("s̏I[łȂ̂ɂւ炸f~^ꂸɍsIĂ܂܂B");
                }
                break;
            }
            if (iRead == delimiter) {
                if (isEndOfLine) {
                    throw new BlancoCsvIOException("s̏I[łׂɂւ炸f~^܂B");
                }
                break;
            } else {
                buffer.append((char) iRead);
            }
        }
        return buffer.toString();
    }

    /**
     * ^ꂽReader _uNI[gt ^ꂽf~^găg[No܂
     *
     * @param delimiter f~^
     * @param isEndOfLine s̏I[ł̂ǂtO
     * @return oꂽg[NB͂⃊[_̏ꍇɂnullԂ܂B
     * @throws BlancoCsvIOException _uNI[g̕svȂǗ^ꂽɊւOꍇB
     * @throws IOException o͗OꍇB
     */
    protected String readTokenWithQuote(final char delimiter, final boolean isEndOfLine) throws BlancoCsvIOException, IOException {
        final StringBuffer buffer = new StringBuffer();
        boolean isStringStarted = false;
        boolean isStringEnded = false;
        for (;;) {
            int iRead = fLineReader.read();
            if (iRead < 0) {
                // I[ɓBB
                if (isStringStarted && isStringEnded == false) {
                    // _uNI[gB
                    fLine = fReader.readLine();
                    if (fLine == null) {
                        // _uNI[gł̂ɂ炸t@C̏I[ɓB܂B
                        break;
                    } else {
                        fLineReader = new StringReader(fLine);
                        continue;
                    }
                } else if (isEndOfLine == false) {
                    throw new BlancoCsvIOException("s̏I[łȂ̂ɂւ炸f~^ꂸɍsIĂ܂܂B");
                }
                break;
            }
            if (isStringStarted == false) {
                if (iRead != '"') {
                    throw new BlancoCsvIOException("_uNI[gŕ񂪊Jn܂B");
                }
                // _uNI[e[V͓ǂݔ΂܂B
                isStringStarted = true;
            } else if (isStringEnded) {
                if (iRead == delimiter) {
                    // I[ɓB܂B
                    if (isEndOfLine) {
                        throw new BlancoCsvIOException("s̏I[łׂɂւ炸f~^܂B");
                    }
                    break;
                }
                throw new BlancoCsvIOException("_uNI[gɂ镶Iɕ񂪗^܂B");
            } else {
                // ʏ̕GA
                if (iRead == '"') {
                    // _uNI[gd˂ꂽ̂łȂǂ`FbN܂B
                    fLineReader.mark(1);
                    if (fLineReader.read() == '"') {
                        // GXP[vꂽ_uNI[g
                        // 2łЂƂȂ̂ŁAЂƂ͓ǂݎ̂Ă܂B
                        buffer.append((char) iRead);
                    } else {
                        // _uNI[gɂGXP[vł͂܂łB
                        // ͏I[ӖĂ܂B
                        fLineReader.reset();
                        isStringEnded = true;
                    }
                } else {
                    buffer.append((char) iRead);
                }
            }
        }
        if (isStringStarted == false) {
            throw new BlancoCsvIOException("_uNI[gKvȕł̂Ƀ_uNI[gɂJn܂łB");
        }
        if (isStringEnded == false) {
            throw new BlancoCsvIOException("_uNI[gKvȕł̂Ƀ_uNI[gɂI܂łB");
        }
        return buffer.toString();
    }
}

==> blancoNLpackGenerator-0.0.94-src.zip: src/blanco/nlpack/generator/csv/io/BlancoNLpackGeneratorResultDetailCsvReader.java <==
/*
 * ̃\[XR[h blanco FrameworkɂĎĂ܂B
 */
package blanco.nlpack.generator.csv.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import blanco.nlpack.generator.csv.record.BlancoNLpackGeneratorResultDetailCsvRecord;
import blanco.nlpack.generator.csv.util.BlancoCsvRuntimeUtil;

/**
 * t@C`[BlancoNLpackGeneratorResultDetail/blancoNLpackGenerator IɗpCSVt@CB|tOg̓̓t@CƂĂp܂BȂA|tOg̓t@CƂėpꍇɂ́ABundleURI, Input, Output ȊO̗͖܂B]̃[_NX
 *
 * ̃NXblancoCsvt@C`玩ꂽ[_NXłB<br>
 * @1.񒷃`FbNɂ́AftHgGR[fBO𗘗p܂B<br>
 * @2.NX̗p͕K close()ĂяoĂB<br>
 */
public class BlancoNLpackGeneratorResultDetailCsvReader {
    /**
     * [_IuWFNg
     *
     * IɎۂɓ͂s̃[_L܂B
     */
    private BufferedReader fReader;

    /**
     * ̍sJE^
     *
     * ݏsĂs̈ʒuL܂B
     */
    private long fLineCounter = 0;

    /**
     * ݏ̍sf[^
     */
    private String fLine;

    /**
     * ݏ̍ŝ߂̃[_
     */
    private StringReader fLineReader;

    /**
     * CSV[_NX̃RXgN^
     *
     * ^ꂽ[_ƘA܂B
     *
     * @param arg A郊[_
     */
    public BlancoNLpackGeneratorResultDetailCsvReader(final BufferedReader arg) {
        fReader = arg;
    }

    /**
     * ̈sǂݍ݂܂
     *
     * @return sIuWFNgԂ܂B͂sꍇɂ nullԂ܂B
     * @throws BlancoCsvIOException ̓f[^sȏꍇȂǁB
     * @throws IOException A惊[_ňُ킪ꍇB
     */
    public BlancoNLpackGeneratorResultDetailCsvRecord readRecord() throws BlancoCsvIOException, IOException {
        fLine = fReader.readLine();
        if (fLine == null) {
            // t@C̏I[ɓB܂B
            return null;
        }
        fLineCounter++;
        final BlancoNLpackGeneratorResultDetailCsvRecord record = new BlancoNLpackGeneratorResultDetailCsvRecord();
        fLineReader = new StringReader(fLine);
        String tokenString = null;

        // ڔԍ[1]ږ[BundleUri/URI]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 1ږځBڔԍ[1] ږ[BundleUri/URI]̏ɂĕsȏI[m܂B");
        }
        // K{ځB
        if (tokenString.length() == 0) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 1ږځBڔԍ[1] ږ[BundleUri/URI]̏ɂĕK{ڂɒlĂȂƂm܂B");
        }
        record.setBundleUri(tokenString);

        // ڔԍ[2]ږ[Action/Action]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 2ږځBڔԍ[2] ږ[Action/Action]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setAction(tokenString);
        } else {
            if (tokenString.getBytes().length < 3) {
                throw new BlancoCsvIOException("" + fLineCounter + "s 2ږځBڔԍ[2] ږ[Action/Action]̏ɂMIN(3)Zl[" + tokenString + "]m܂B");
            }
            record.setAction(tokenString);
        }

        // ڔԍ[3]ږ[Key/Key]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 3ږځBڔԍ[3] ږ[Key/Key]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setKey(tokenString);
        } else {
            if (tokenString.getBytes().length < 1) {
                throw new BlancoCsvIOException("" + fLineCounter + "s 3ږځBڔԍ[3] ږ[Key/Key]̏ɂMIN(1)Zl[" + tokenString + "]m܂B");
            }
            record.setKey(tokenString);
        }

        // ڔԍ[4]ږ[Input/Input]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 4ږځBڔԍ[4] ږ[Input/Input]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setInput(tokenString);
        } else {
            record.setInput(tokenString);
        }

        // ڔԍ[5]ږ[Output/Output]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', false);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 5ږځBڔԍ[5] ږ[Output/Output]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setOutput(tokenString);
        } else {
            record.setOutput(tokenString);
        }

        // ڔԍ[6]ږ[Note/Note]
        // NI[glǂݍ݂s܂B
        tokenString = readTokenWithQuote(',', true);
        if (tokenString == null) {
            throw new BlancoCsvIOException("" + fLineCounter + "s 6ږځBڔԍ[6] ږ[Note/Note]̏ɂĕsȏI[m܂B");
        }
        // CӍځB
        if (tokenString.length() == 0) {
            // K{ڂł͂Ȃ񍀖ڂɒ0̒lǂݍ܂ꂽꍇɂ́Aǂݍ݌̒lƂĂ0̕Ƃ܂B
            record.setNote(tokenString);
        } else {
            record.setNote(tokenString);
        }
        return record;
    }

    /**
     * IɕێĂ郊[_Ԃ܂B
     *
     * @return IɕێĂ郊[_B
     */
    public BufferedReader getReader() {
        return fReader;
    }

    /**
     * ݏ̍sJE^Ԃ܂B
     *
     * @return ݏ̍sJE^B
     */
    public long getLineCounter() {
        return fLineCounter;
    }

    /**
     * ̃[_܂B
     *
     * ֘AÂĂ[_ɑ΂Ăclose()Ăяo܂B
     *
     * @throws IOException ֘AÂĂ[_close()ɎsꍇB
     */
    public void close() throws IOException {
        fReader.close();
    }

    /**
     * ^ꂽReader ^ꂽf~^găg[No܂
     *
     * @param delimiter f~^
     * @param isEndOfLine s̏I[ł̂ǂtO
     * @return oꂽg[NB͂⃊[_̏ꍇɂnullԂ܂B
     * @throws BlancoCsvIOException ڐȂȂǗ^ꂽɊւOꍇB
     * @throws IOException o͗OꍇB
     */
    protected String readToken(final char delimiter, final boolean isEndOfLine) throws BlancoCsvIOException, IOException {
        final StringBuffer buffer = new StringBuffer();
        for (;;) {
            final int iRead = fLineReader.read();
            if (iRead < 0) {
                // I[ɓBB
                if (isEndOfLine == false) {
                    throw new BlancoCsvIOException("s̏I[łȂ̂ɂւ炸f~^ꂸɍsIĂ܂܂B");
                }
                break;
            }
            if (iRead == delimiter) {
                if (isEndOfLine) {
                    throw new BlancoCsvIOException("s̏I[łׂɂւ炸f~^܂B");
                }
                break;
            } else {
                buffer.append((char) iRead);
            }
        }
        return buffer.toString();
    }

    /**
     * ^ꂽReader _uNI[gt ^ꂽf~^găg[No܂
     *
     * @param delimiter f~^
     * @param isEndOfLine s̏I[ł̂ǂtO
     * @return oꂽg[NB͂⃊[_̏ꍇɂnullԂ܂B
     * @throws BlancoCsvIOException _uNI[g̕svȂǗ^ꂽɊւOꍇB
     * @throws IOException o͗OꍇB
     */
    protected String readTokenWithQuote(final char delimiter, final boolean isEndOfLine) throws BlancoCsvIOException, IOException {
        final StringBuffer buffer = new StringBuffer();
        boolean isStringStarted = false;
        boolean isStringEnded = false;
        for (;;) {
            int iRead = fLineReader.read();
            if (iRead < 0) {
                // I[ɓBB
                if (isStringStarted && isStringEnded == false) {
                    // sóB
                    buffer.append('\n');
                    // _uNI[gB
                    fLine = fReader.readLine();
                    if (fLine == null) {
                        // _uNI[gł̂ɂ炸t@C̏I[ɓB܂B
                        break;
                    } else {
                        fLineReader = new StringReader(fLine);
                        continue;
                    }
                } else if (isEndOfLine == false) {
                    throw new BlancoCsvIOException("s̏I[łȂ̂ɂւ炸f~^ꂸɍsIĂ܂܂B");
                }
                break;
            }
            if (isStringStarted == false) {
                if (iRead != '"') {
                    throw new BlancoCsvIOException("_uNI[gŕ񂪊Jn܂B");
                }
                // _uNI[e[V͓ǂݔ΂܂B
                isStringStarted = true;
            } else if (isStringEnded) {
                if (iRead == delimiter) {
                    // I[ɓB܂B
                    if (isEndOfLine) {
                        throw new BlancoCsvIOException("s̏I[łׂɂւ炸f~^܂B");
                    }
                    break;
                }
                throw new BlancoCsvIOException("_uNI[gɂ镶Iɕ񂪗^܂B");
            } else {
                // ʏ̕GA
                if (iRead == '"') {
                    // _uNI[gd˂ꂽ̂łȂǂ`FbN܂B
                    fLineReader.mark(1);
                    if (fLineReader.read() == '"') {
                        // GXP[vꂽ_uNI[g
                        // 2łЂƂȂ̂ŁAЂƂ͓ǂݎ̂Ă܂B
                        buffer.append((char) iRead);
                    } else {
                        // _uNI[gɂGXP[vł͂܂łB
                        // ͏I[ӖĂ܂B
                        fLineReader.reset();
                        isStringEnded = true;
                    }
                } else {
                    buffer.append((char) iRead);
                }
            }
        }
        if (isStringStarted == false) {
            throw new BlancoCsvIOException("_uNI[gKvȕł̂Ƀ_uNI[gɂJn܂łB");
        }
        if (isStringEnded == false) {
            throw new BlancoCsvIOException("_uNI[gKvȕł̂Ƀ_uNI[gɂI܂łB");
        }
        return buffer.toString();
    }
}
